# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: arg_services/base/v1/base.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='arg_services/base/v1/base.proto',
  package='arg_services.base.v1',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1f\x61rg_services/base/v1/base.proto\x12\x14\x61rg_services.base.v1*F\n\x08Language\x12\x18\n\x14LANGUAGE_UNSPECIFIED\x10\x00\x12\x0f\n\x0bLANGUAGE_EN\x10\x01\x12\x0f\n\x0bLANGUAGE_DE\x10\x02\x62\x06proto3'
)

_LANGUAGE = _descriptor.EnumDescriptor(
  name='Language',
  full_name='arg_services.base.v1.Language',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LANGUAGE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANGUAGE_EN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANGUAGE_DE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=57,
  serialized_end=127,
)
_sym_db.RegisterEnumDescriptor(_LANGUAGE)

Language = enum_type_wrapper.EnumTypeWrapper(_LANGUAGE)
LANGUAGE_UNSPECIFIED = 0
LANGUAGE_EN = 1
LANGUAGE_DE = 2


DESCRIPTOR.enum_types_by_name['Language'] = _LANGUAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc

from .entailment_pb2 import *
class EntailmentServiceStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    def Entailment(self,
        request: global___EntailmentRequest,
    ) -> global___EntailmentResponse: ...


class EntailmentServiceServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def Entailment(self,
        request: global___EntailmentRequest,
        context: grpc.ServicerContext,
    ) -> global___EntailmentResponse: ...


def add_EntailmentServiceServicer_to_server(servicer: EntailmentServiceServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import arg_services.base.v1.base_pb2
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

global___EmbeddingLevel = EmbeddingLevel
class _EmbeddingLevel(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[EmbeddingLevel.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    EMBEDDING_LEVEL_UNSPECIFIED = EmbeddingLevel.V(0)
    EMBEDDING_LEVEL_DOCUMENT = EmbeddingLevel.V(1)
    EMBEDDING_LEVEL_TOKENS = EmbeddingLevel.V(2)
    EMBEDDING_LEVEL_SENTENCES = EmbeddingLevel.V(3)
class EmbeddingLevel(metaclass=_EmbeddingLevel):
    V = typing.NewType('V', builtins.int)
EMBEDDING_LEVEL_UNSPECIFIED = EmbeddingLevel.V(0)
EMBEDDING_LEVEL_DOCUMENT = EmbeddingLevel.V(1)
EMBEDDING_LEVEL_TOKENS = EmbeddingLevel.V(2)
EMBEDDING_LEVEL_SENTENCES = EmbeddingLevel.V(3)

global___Pooling = Pooling
class _Pooling(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Pooling.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    POOLING_UNSPECIFIED = Pooling.V(0)
    POOLING_MEAN = Pooling.V(1)
    POOLING_MAX = Pooling.V(2)
    POOLING_MIN = Pooling.V(3)
    POOLING_SUM = Pooling.V(4)
    POOLING_FIRST = Pooling.V(5)
    POOLING_LAST = Pooling.V(6)
class Pooling(metaclass=_Pooling):
    V = typing.NewType('V', builtins.int)
POOLING_UNSPECIFIED = Pooling.V(0)
POOLING_MEAN = Pooling.V(1)
POOLING_MAX = Pooling.V(2)
POOLING_MIN = Pooling.V(3)
POOLING_SUM = Pooling.V(4)
POOLING_FIRST = Pooling.V(5)
POOLING_LAST = Pooling.V(6)

global___EmbeddingType = EmbeddingType
class _EmbeddingType(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[EmbeddingType.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    EMBEDDING_TYPE_UNSPECIFIED = EmbeddingType.V(0)
    EMBEDDING_TYPE_SPACY = EmbeddingType.V(1)
    EMBEDDING_TYPE_USE = EmbeddingType.V(2)
    EMBEDDING_TYPE_SBERT = EmbeddingType.V(3)
class EmbeddingType(metaclass=_EmbeddingType):
    V = typing.NewType('V', builtins.int)
EMBEDDING_TYPE_UNSPECIFIED = EmbeddingType.V(0)
EMBEDDING_TYPE_SPACY = EmbeddingType.V(1)
EMBEDDING_TYPE_USE = EmbeddingType.V(2)
EMBEDDING_TYPE_SBERT = EmbeddingType.V(3)

class DocBinRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LANGUAGE_FIELD_NUMBER: builtins.int
    TEXTS_FIELD_NUMBER: builtins.int
    SPACY_MODEL_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    EMBEDDING_LEVELS_FIELD_NUMBER: builtins.int
    EMBEDDING_MODELS_FIELD_NUMBER: builtins.int
    language: arg_services.base.v1.base_pb2.Language.V = ...
    texts: google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text] = ...
    spacy_model: typing.Text = ...
    attributes: google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text] = ...
    embedding_levels: google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EmbeddingLevel.V] = ...

    @property
    def embedding_models(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EmbeddingModel]: ...

    def __init__(self,
        *,
        language : arg_services.base.v1.base_pb2.Language.V = ...,
        texts : typing.Optional[typing.Iterable[typing.Text]] = ...,
        spacy_model : typing.Text = ...,
        attributes : typing.Optional[typing.Iterable[typing.Text]] = ...,
        embedding_levels : typing.Optional[typing.Iterable[global___EmbeddingLevel.V]] = ...,
        embedding_models : typing.Optional[typing.Iterable[global___EmbeddingModel]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attributes",b"attributes",u"embedding_levels",b"embedding_levels",u"embedding_models",b"embedding_models",u"language",b"language",u"spacy_model",b"spacy_model",u"texts",b"texts"]) -> None: ...
global___DocBinRequest = DocBinRequest

class DocBinResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DOCBIN_FIELD_NUMBER: builtins.int
    docbin: builtins.bytes = ...

    def __init__(self,
        *,
        docbin : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"docbin",b"docbin"]) -> None: ...
global___DocBinResponse = DocBinResponse

class VectorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LANGUAGE_FIELD_NUMBER: builtins.int
    TEXTS_FIELD_NUMBER: builtins.int
    SPACY_MODEL_FIELD_NUMBER: builtins.int
    EMBEDDING_LEVELS_FIELD_NUMBER: builtins.int
    EMBEDDING_MODELS_FIELD_NUMBER: builtins.int
    language: arg_services.base.v1.base_pb2.Language.V = ...
    texts: google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text] = ...
    spacy_model: typing.Text = ...
    embedding_levels: google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EmbeddingLevel.V] = ...

    @property
    def embedding_models(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EmbeddingModel]: ...

    def __init__(self,
        *,
        language : arg_services.base.v1.base_pb2.Language.V = ...,
        texts : typing.Optional[typing.Iterable[typing.Text]] = ...,
        spacy_model : typing.Text = ...,
        embedding_levels : typing.Optional[typing.Iterable[global___EmbeddingLevel.V]] = ...,
        embedding_models : typing.Optional[typing.Iterable[global___EmbeddingModel]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"embedding_levels",b"embedding_levels",u"embedding_models",b"embedding_models",u"language",b"language",u"spacy_model",b"spacy_model",u"texts",b"texts"]) -> None: ...
global___VectorRequest = VectorRequest

class VectorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DOCUMENT_FIELD_NUMBER: builtins.int
    TOKENS_FIELD_NUMBER: builtins.int
    SENTENCES_FIELD_NUMBER: builtins.int

    @property
    def document(self) -> global___Vector: ...

    @property
    def tokens(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Vector]: ...

    @property
    def sentences(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Vector]: ...

    def __init__(self,
        *,
        document : typing.Optional[global___Vector] = ...,
        tokens : typing.Optional[typing.Iterable[global___Vector]] = ...,
        sentences : typing.Optional[typing.Iterable[global___Vector]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"document",b"document"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"document",b"document",u"sentences",b"sentences",u"tokens",b"tokens"]) -> None: ...
global___VectorResponse = VectorResponse

class Vector(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VECTOR_FIELD_NUMBER: builtins.int
    vector: google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float] = ...

    def __init__(self,
        *,
        vector : typing.Optional[typing.Iterable[builtins.float]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"vector",b"vector"]) -> None: ...
global___Vector = Vector

class EmbeddingModel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MODEL_TYPE_FIELD_NUMBER: builtins.int
    MODEL_NAME_FIELD_NUMBER: builtins.int
    POOLING_FIELD_NUMBER: builtins.int
    model_type: global___EmbeddingType.V = ...
    model_name: typing.Text = ...
    pooling: global___Pooling.V = ...

    def __init__(self,
        *,
        model_type : global___EmbeddingType.V = ...,
        model_name : typing.Text = ...,
        pooling : global___Pooling.V = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"model_name",b"model_name",u"model_type",b"model_type",u"pooling",b"pooling"]) -> None: ...
global___EmbeddingModel = EmbeddingModel



import contextlib
import multiprocessing
import socket
import traceback
import typing as t
from concurrent import futures
from operator import attrgetter

import grpc


def handle_except(ex: Exception, ctx: grpc.ServicerContext) -> None:
    """Handler that can be called when handling an exception.

    It will pass the traceback to the gRPC client and abort the context.

    Args:
        ex: Exception that occured.
        ctx: Current gRPC context.
    """

    msg = "".join(traceback.TracebackException.from_exception(ex).format())
    ctx.abort(grpc.StatusCode.UNKNOWN, msg)


def require(
    attrs: t.Collection[str],
    obj: object,
    ctx: grpc.ServicerContext,
    parent: t.Optional[str] = None,
) -> None:
    """Verify that all required arguments are supplied by the client.

    If arguments are missing, the context will be aborted

    Args:
        attrs: Names of the required parameters.
        obj: Current request message (e.g., a subclass of google.protobuf.message.Message).
        ctx: Current gRPC context.
        parent: Name of the parent message. Only used to compose more helpful error.
    """
    if not parent:
        parent = "request"

    func = attrgetter(*attrs)
    attr_result = func(obj)

    if len(attrs) == 1:
        attr_result = [attr_result]

    if not all(attr_result):
        ctx.abort(
            grpc.StatusCode.INVALID_ARGUMENT,
            f"The message '{parent}' requires the following attributes: {attrs}.",
        )


def _serve_single(
    bind_address: str,
    add_services: t.Callable[[grpc.Server], None],
    threads: int,
    reuse_port: bool,
):
    """Helper function to start a server for a single process.

    Args:
        bind_address: Complete address consisting of hostname and port (e.g., `127.0.0.1:8000`)
        add_services: Function to inject the gRPC services into the server instance.
        threads: Number of workers per process.
        reuse_port: If using multiple processes, the port has to be reused.
    """

    server = grpc.server(
        futures.ThreadPoolExecutor(max_workers=threads),
        options=(("grpc.so_reuseport", 1 if reuse_port else 0),),
    )
    add_services(server)

    server.add_insecure_port(bind_address)
    server.start()
    server.wait_for_termination()


def serve(
    host: str,
    port: int,
    add_services: t.Callable[[grpc.Server], None],
    processes: int = 1,
    threads: int = 1,
):
    """Serve one or multiple gRPC services, optionally using multiprocessing.

    Args:
        host: Hostname of the server (e.g., `127.0.0.1`)
        port: Port of the server. Use `0` to let the server determine an open port.
        add_services: Function to inject the gRPC services into the server instance.
        processes: If `processes > 1`, multiprocessing with a reused port is applied.
        threads: Number of workers per process.

    Raises:
        ValueError: If `processes < 0` is given.
    """

    if processes < 0:
        raise ValueError("No negative number of processes allowed.")

    reuse_port = processes > 1

    with _reserve_port(port, reuse_port) as actual_port:
        bind_address = f"{host}:{actual_port}"
        print(f"Serving on {bind_address}")
        args = (bind_address, add_services, threads, reuse_port)

        if processes == 1:
            _serve_single(*args)
        else:
            workers = []

            for _ in range(processes):
                worker = multiprocessing.Process(target=_serve_single, args=args)
                worker.start()
                workers.append(worker)

            for worker in workers:
                worker.join()


@contextlib.contextmanager
def _reserve_port(
    port: int = 0, reuse_port: bool = False
) -> t.Generator[int, None, None]:
    """Find and reserve a port for all subprocesses to use.

    Args:
        port: Desired port. If `0`, the method will automatically determine a free port.
        reuse_port: If using multiple processes, set this to `True`.
            Only affects multiprocessing, not multithreading.

    Yields:
        Port that has been reserved

    Raises:
        RuntimeError: If the port could not be reused
    """

    sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)

    if reuse_port:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)

        if sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT) == 0:
            raise RuntimeError("Failed to set SO_REUSEPORT.")

    sock.bind(("", port))

    try:
        yield sock.getsockname()[1]
    finally:
        sock.close()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc

from .nlp_pb2 import *
class NlpServiceStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    def DocBin(self,
        request: global___DocBinRequest,
    ) -> global___DocBinResponse: ...

    def Vector(self,
        request: global___VectorRequest,
    ) -> global___VectorResponse: ...


class NlpServiceServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def DocBin(self,
        request: global___DocBinRequest,
        context: grpc.ServicerContext,
    ) -> global___DocBinResponse: ...

    @abc.abstractmethod
    def Vector(self,
        request: global___VectorRequest,
        context: grpc.ServicerContext,
    ) -> global___VectorResponse: ...


def add_NlpServiceServicer_to_server(servicer: NlpServiceServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

global___Language = Language
class _Language(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Language.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    LANGUAGE_UNSPECIFIED = Language.V(0)
    LANGUAGE_EN = Language.V(1)
    LANGUAGE_DE = Language.V(2)
class Language(metaclass=_Language):
    V = typing.NewType('V', builtins.int)
LANGUAGE_UNSPECIFIED = Language.V(0)
LANGUAGE_EN = Language.V(1)
LANGUAGE_DE = Language.V(2)
